@@ -42,8 +42,8 @@ void show_game_over_dialog() {
   gchar* message;
 
   if (game_won)
-    message = g_strdup_printf ("<b>%s\n\n%s</b>", _("Congratulations."),
-                                _("You have won!!!"));
+    message = g_strdup_printf ("<b>%s\n\n%s</b>", _("Congratulations!!!"),
+                                _("You have won."));
   else
     message = g_strdup_printf ("<b>%s</b>", _("There are no more moves."));
 
@@ -61,7 +61,7 @@ void show_game_over_dialog() {
 			    NULL);
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			  GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE,
-                          _("New Game"), GTK_RESPONSE_ACCEPT,
+                          _("_New Game"), GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
@@ -191,7 +191,7 @@ void show_select_game_dialog (void)
     g_object_unref (G_OBJECT (list));
     
     renderer = gtk_cell_renderer_text_new();
-    column = gtk_tree_view_column_new_with_attributes(_("Rules"),
+    column = gtk_tree_view_column_new_with_attributes(NULL,
                                                       renderer,
                                                       "text", 0,
                                                       NULL);
@@ -229,7 +229,10 @@ void show_select_game_dialog (void)
  
     games_file_list_for_each (files, (GFunc) build_list, list);
 
-    g_free (files);
+    g_object_unref (files);
+
+    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list),
+					  0, GTK_SORT_ASCENDING);
 
     gtk_dialog_set_default_response ( GTK_DIALOG (dialog), GTK_RESPONSE_OK );
 
