@@ -7,42 +7,43 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <glib-object.h>
+#include <cairo.h>
+#include <math.h>
+#include <string.h>
 
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
 #include "gtkgridboard.h"
 
+#undef DO_PROFILE
+
+#ifdef DO_PROFILE
+#include <sys/time.h>
+
+/* A temporary profiling counter. Only supports one instance.*/
+long int counter;
+#endif
+
 static GtkWidgetClass *parent_class=NULL;
 static guint gridboard_signals[LAST_SIGNAL] = { 0 };
 
-/* Save the flip pixmaps timeout so we can kill it at the games end. */
-guint timeoutid = 0;
-
 /* prototypes for private functions */
-static gint ** make_array(int width, int height); 
-static void gtk_gridboard_draw_box(GtkGridBoard *widget, gint x, gint y); 
-static void gtk_gridboard_draw_box_with_gc(GtkGridBoard * widget, GdkGC * gc, gint x, gint y);
-static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard);
+static void gtk_gridboard_repaint(GtkGridBoard * gridboard);
+static gint ** make_array(int width, int height, int fill); 
 static gint get_pixmap_num(int piece);
-static gint gtk_gridboard_button_press(GtkWidget *widget, GdkEventButton *event) ;
-static gint gtk_gridboard_expose(GtkWidget * widget, GdkEventExpose * event);
+static gint gtk_gridboard_button_press(GtkWidget *widget, 
+				       GdkEventButton *event) ;
+static gboolean gtk_gridboard_expose(GtkWidget * widget, 
+				     GdkEventExpose * event);
+static void gtk_gridboard_size_allocate(GtkWidget * widget, 
+					GtkAllocation * allocation);
 static void gtk_gridboard_class_init(GtkGridBoardClass * class);
 static void gtk_gridboard_finalize(GObject * object);
 static void gtk_gridboard_init(GtkGridBoard * gridboard);
 static void gtk_gridboard_realize(GtkWidget * widget);
-static void gtk_gridboard_size_allocate(GtkWidget * widget, GtkAllocation * allocation);
 static void gtk_gridboard_size_request(GtkWidget * widget, GtkRequisition * req);
-static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint x, gint y);
 static gint gtk_gridboard_flip_pixmaps(gpointer data);
 
 /* init functions */
@@ -60,8 +61,8 @@ static void gtk_gridboard_class_init(GtkGridBoardClass * class) {
         object_class->finalize = gtk_gridboard_finalize;
         widget_class->realize = gtk_gridboard_realize;
         widget_class->expose_event = gtk_gridboard_expose;
-        widget_class->size_request = gtk_gridboard_size_request;
-        widget_class->size_allocate = gtk_gridboard_size_allocate;
+	widget_class->size_allocate = gtk_gridboard_size_allocate;
+	widget_class->size_request = gtk_gridboard_size_request;
         widget_class->button_press_event = gtk_gridboard_button_press;
         
         gridboard_signals[BOXCLICKED] = g_signal_new ("boxclicked",
@@ -74,31 +75,34 @@ static void gtk_gridboard_class_init(GtkGridBoardClass * class) {
 }
 
 static void gtk_gridboard_init(GtkGridBoard * gridboard) {
-        gridboard->tileset=NULL;
-        gridboard->showgrid=FALSE;
-        gridboard->policy=GTK_UPDATE_CONTINUOUS;
+        gridboard->backing_store = NULL;
+	gridboard->tiles_pixbuf = NULL;
+	gridboard->tiles_scale = 1;
         gridboard->pixmaps=NULL;
+	gridboard->timeoutid = 0;
         gridboard->tileheight=TILEHEIGHT;
         gridboard->tilewidth=TILEWIDTH;
         gridboard->width=0;
         gridboard->height=0;
 	gridboard->statelist=NULL;
+	gridboard->theme = gtk_gridboard_themes;
 }
 
-GtkWidget * gtk_gridboard_new(gint width, gint height, char * tileset) {
+GtkWidget * gtk_gridboard_new(gint width, gint height, char * theme) {
         GtkGridBoard * gridboard=g_object_new(gtk_gridboard_get_type(), NULL);
 
         gridboard->width=width;
         gridboard->height=height;
-        gridboard->tileset=tileset;
 
-        gridboard->board=make_array(width, height);
-        gridboard->pixmaps=make_array(width, height);
-        gridboard->selected=make_array(width, height);
-
-        gtk_gridboard_load_pixmaps(gridboard);
+        gridboard->board    = make_array (width, height, 0);
+        gridboard->pixmaps  = make_array (width, height, 0);
+        gridboard->selected = make_array (width, height, SELECTED_NONE);
+	gridboard->changed  = make_array (width, height, TRUE);
+	
         gtk_gridboard_set_animate(gridboard, TRUE);
         gtk_gridboard_set_visibility(gridboard, TRUE);
+
+	gtk_gridboard_set_theme (gridboard, theme);
         
         return GTK_WIDGET(gridboard);
         
@@ -141,8 +145,8 @@ static gint gtk_gridboard_button_press(GtkWidget *widget,
    
    gridboard = GTK_GRIDBOARD (widget);
 
-   x = event->x / gridboard->tilewidth;
-   y = event->y / gridboard->tileheight;
+   x = event->x * gridboard->width / widget->allocation.width;
+   y = event->y * gridboard->height / widget->allocation.height;
 
    x = CLAMP (x, 0, gridboard->width - 1);
    y = CLAMP (y, 0, gridboard->height - 1);
@@ -156,18 +160,40 @@ static gint gtk_gridboard_button_press(GtkWidget *widget,
 static gboolean gtk_gridboard_expose(GtkWidget *widget,
 				     GdkEventExpose * event)
  {
-        g_return_val_if_fail (GTK_IS_GRIDBOARD(widget), FALSE);
+        GtkGridBoard *gridboard;
 
-        /* multiple exposures, do only the last one */
-        if (event->count>0) return FALSE;
+	if (!GTK_WIDGET_DRAWABLE (widget))
+	  return FALSE;
 
-	/* FIXME: We should only redraw the bit we need to. */
+	gridboard = GTK_GRIDBOARD (widget);
 
-        gtk_gridboard_paint(GTK_GRIDBOARD (widget));
+	gdk_draw_drawable (widget->window, widget->style->black_gc,
+			   gridboard->backing_store, event->area.x,
+			   event->area.y, event->area.x, event->area.y,
+			   event->area.width, event->area.height);
 
         return FALSE;
 }
 
+static void gtk_gridboard_size_allocate (GtkWidget * widget,
+					 GtkAllocation *allocation)
+{
+  GtkGridBoard *gridboard = GTK_GRIDBOARD (widget);
+
+  widget->allocation = *allocation;
+ 
+  if (gridboard->backing_store != NULL) {
+    g_object_unref (gridboard->backing_store);
+    gridboard->backing_store = NULL;
+  }
+
+  if (GTK_WIDGET_REALIZED (widget)) {
+    gdk_window_move_resize (widget->window, allocation->x, allocation->y,
+			    allocation->width, allocation->height);
+    gtk_gridboard_paint (gridboard);
+  }
+}
+
 static void gtk_gridboard_realize(GtkWidget * widget) 
 {
         GdkWindowAttr attributes;
@@ -198,6 +224,8 @@ static void gtk_gridboard_realize(GtkWidget * widget)
         gdk_window_set_user_data (widget->window, widget);
 
         gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
+	
+	gtk_gridboard_paint (GTK_GRIDBOARD (widget));
 }
 
 static void gtk_gridboard_finalize(GObject * object) {
@@ -208,9 +236,11 @@ static void gtk_gridboard_finalize(GObject * object) {
 
         gridboard = GTK_GRIDBOARD(object);
 
-        g_free(gridboard->board);
-        g_free(gridboard->pixmaps);
-        g_free(gridboard->selected);
+	g_object_unref (gridboard->backing_store);
+        g_free (gridboard->board);
+        g_free (gridboard->pixmaps);
+        g_free (gridboard->selected);
+	g_free (gridboard->changed);
 
         if (G_OBJECT_CLASS (parent_class)->finalize)
                 (* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -227,72 +257,92 @@ static void gtk_gridboard_size_request(GtkWidget * widget, GtkRequisition * req)
         req->height=gridboard->height*gridboard->tileheight + 1;
 }
 
-/* this isn't really used because the window is not resizable */
-static void gtk_gridboard_size_allocate(GtkWidget *widget,
-					GtkAllocation * allocation) 
-{
-        GtkGridBoard *gridboard; 
-
-        g_return_if_fail(GTK_IS_GRIDBOARD(widget));
-  
-	gridboard = GTK_GRIDBOARD (widget);
-
-	widget->allocation = *allocation;
-
-        if (GTK_WIDGET_REALIZED (widget)) {
-                gdk_window_move_resize (widget->window,
-                        allocation->x, 
-                        allocation->y,
-                        allocation->width, 
-                        allocation->height);
-        }
-}
-
-
 /* drawing functions */
+
 static gint get_pixmap_num(int piece) {
         if (piece==BLACK) return BLACK_PIXMAP;
         if (piece==WHITE) return WHITE_PIXMAP;
         return EMPTY_PIXMAP;
 }
 
-void gtk_gridboard_paint(GtkGridBoard * gridboard) {
-        int x, y;
-        int piecepic;
+void gtk_gridboard_paint (GtkGridBoard *gridboard)
+{
+  GtkWidget *widget;
 
-	if (!gridboard->visibility) return;
+  g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+  
+  widget = GTK_WIDGET (gridboard);
+  g_return_if_fail (GTK_WIDGET_REALIZED (widget));
 
-        for (x=0; x<gridboard->height; x++) {
-                for (y=0; y<gridboard->width; y++) {
-                        piecepic=get_pixmap_num(gridboard->board[x][y]);
-                        gtk_gridboard_draw_pixmap(gridboard, piecepic, x, y);
-                }
-        }
-	/* FIXME: There has to be a better way. */
-        for (x=0; x<gridboard->height; x++) {
-                for (y=0; y<gridboard->width; y++) {
-		  if (gridboard->selected[x][y] != SELECTED_NONE)
-			    gtk_gridboard_draw_box(gridboard, x, y);
-                }
-        }
+  gtk_widget_queue_draw (widget);
+
+  gtk_gridboard_repaint (gridboard);
+}
+
+static void gtk_gridboard_repaint(GtkGridBoard * gridboard) {
+  int x, y;
+  double phase;
+  GtkWidget *widget;
+  gdouble scale;
+  cairo_t *cx;
+  
+#ifdef DO_PROFILE
+  struct timeval tv;
+#endif
+  
+  g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+  if (!gridboard->visibility) return;
+  
+  widget = GTK_WIDGET (gridboard);
+  g_return_if_fail (GTK_WIDGET_REALIZED (widget));
+
+  if (gridboard->backing_store == NULL) {
+    gridboard->backing_store = gdk_pixmap_new (widget->window,
+					       widget->allocation.width,
+					       widget->allocation.height, 
+					       -1);
+    for (x=0; x<gridboard->width; x++)
+      for (y=0; y<gridboard->height; y++)
+	gridboard->changed[x][y] = TRUE;
+  }
+  
+  cx = gdk_cairo_create (gridboard->backing_store);
+  cairo_scale (cx, 1.0*widget->allocation.width/gridboard->width,
+	       1.0*widget->allocation.height/gridboard->height);
+    
+#ifdef DO_PROFILE
+  gettimeofday (&tv, NULL);
+  counter = tv.tv_sec * 1000000 + tv.tv_usec;
+#endif
+    
+  scale = gridboard->tiles_scale;
+  
+  for (x=0; x<gridboard->height; x++) {
+    for (y=0; y<gridboard->width; y++) {
+      if (gridboard->changed[x][y]) {
+	gridboard->theme->draw_bg (cx, x, y);
+	gridboard->theme->draw_grid (cx, x, y);
+	if (gridboard->board[x][y] != EMPTY) {
+	  phase = gridboard->pixmaps[x][y];
+	  phase = (phase - 1.0)/30.0;
+	  gridboard->theme->draw_piece (cx, x, y, phase);
+	} else if (gridboard->selected[x][y]) {
+	  gridboard->theme->draw_hilight (cx, x, y);
+	}
+	gridboard->changed[x][y] = FALSE;
+      }
+    }
+  }
+
+  cairo_destroy (cx);
+  
+#ifdef DO_PROFILE
+  gettimeofday (&tv, NULL);
+  g_print ("%ld\n", tv.tv_sec * 1000000 + tv.tv_usec - counter);
+#endif
 }
 
 
-static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint
- x, gint y) {
-        GtkWidget * widget=GTK_WIDGET(gridboard);
-        
-	if (!gridboard->visibility) return;
-
-        gdk_draw_drawable (widget->window,
-                widget->style->fg_gc[widget->state],
-                gridboard->tiles_pixmap,
-                (which % 8) * TILEWIDTH, (which / 8) * TILEHEIGHT,
-                x * TILEWIDTH, y * TILEHEIGHT, TILEWIDTH, TILEHEIGHT);
-
-        /* draw lines around x, y */
-        gtk_gridboard_draw_box(gridboard, x, y);
-}
 
 void gtk_gridboard_set_selection(GtkGridBoard * gridboard, gint type, 
 				 gint x, gint y) 
@@ -303,49 +353,8 @@ void gtk_gridboard_set_selection(GtkGridBoard * gridboard, gint type,
 	g_return_if_fail (x < gridboard->width);
 	g_return_if_fail (y < gridboard->height);
 
-        gridboard->selected[x][y]=type;
-        gtk_gridboard_draw_box(gridboard, x, y);
-}
-
-static void gtk_gridboard_draw_box(GtkGridBoard *gridboard, gint x, gint y) {
-        GtkWidget *widget;
-        GdkGC * gc;
-        int selected;
-        static GdkColor selectcolor={12345, 30208, 41216, 33792};
-
-	if (!gridboard->visibility) return;
-        
-	widget = GTK_WIDGET (gridboard);
-
-        selected=gridboard->selected[x][y];
-        
-        if (selected==SELECTED_NONE) {
-                if (!gridboard->showgrid) {
-                        return;
-                } else {
-                        gc=widget->style->black_gc;
-                }
-        } else if (selected==SELECTED_A) {
-                gc=widget->style->white_gc;
-        } else if (selected==SELECTED_B) {
-                gc=gdk_gc_new(widget->window);
-                gdk_gc_set_foreground(gc, &selectcolor);
-        } else {
-		return;
-	}
-        gtk_gridboard_draw_box_with_gc(gridboard, gc, x, y);
-}
-
-static void gtk_gridboard_draw_box_with_gc(GtkGridBoard * gridboard, 
-					   GdkGC * gc, gint x, gint y) 
-{
-        GtkWidget * widget;
-
-	widget = GTK_WIDGET (gridboard);
-
-        gdk_draw_rectangle(widget->window, gc, FALSE,
-                x*gridboard->tilewidth, y*gridboard->tileheight,
-                gridboard->tilewidth, gridboard->tileheight);
+        gridboard->selected[x][y] = type;
+	gridboard->changed[x][y]  = TRUE;
 }
 
 static gint gtk_gridboard_flip_pixmaps(gpointer data) 
@@ -357,80 +366,66 @@ static gint gtk_gridboard_flip_pixmaps(gpointer data)
         gint pcepm; /* this pixmap should be there */
         gint flipped_tiles;
 
+	if (!GTK_WIDGET_DRAWABLE (GTK_WIDGET (gridboard))) {
+	  return FALSE;
+	}
+
+	flipped_tiles=0;
         for (x=0; x<gridboard->width; x++) {
                 for (y=0; y<gridboard->height; y++) {
+        		/* curpm is the current pixmap being shown.
+			 * pcepm is the target pixmap. We step from
+			 * one to the other during the animation. */
                         curpm=gridboard->pixmaps[x][y];
                         pcepm=get_pixmap_num(gridboard->board[x][y]);
-                        flipped_tiles=0;
-                        if (curpm==pcepm) {
-                                /* nothing needs to be flipped */
+                        if (curpm == pcepm) {
+                                /* Nothing needs to be flipped */
                                 continue;
-                        } else if ((curpm==EMPTY_PIXMAP)||(pcepm==EMPTY_PIXMAP)) {
-                                /* don't animate empty squares */
+                        } else if ((curpm == EMPTY_PIXMAP) ||
+				   (pcepm == EMPTY_PIXMAP)) {
+                                /* Don't animate empty squares */
                                 gridboard->pixmaps[x][y]=pcepm;
                                 flipped_tiles++;
-                        } else if (curpm<pcepm) {
+				gridboard->changed[x][y] = TRUE;
+			} else if (!gridboard->animate) {
+				/* Disabled animations consist of the final frame.*/
+				gridboard->pixmaps[x][y] = pcepm;
+				gridboard->changed[x][y] = TRUE;
+                        } else if (curpm < pcepm) {
                                 gridboard->pixmaps[x][y]++;
                                 flipped_tiles++;
-                        } else if (curpm>pcepm) {
+				gridboard->changed[x][y] = TRUE;
+                        } else { /* curpm > pcepm */
                                 gridboard->pixmaps[x][y]--;
                                 flipped_tiles++;
-                        }
-                        if (flipped_tiles) {
-                                gtk_gridboard_draw_pixmap(
-                                                gridboard,
-                                                gridboard->pixmaps[x][y],
-                                                x, y);
+				gridboard->changed[x][y] = TRUE;
                         }
                 }
         }
 
-        /* destroy the timeout if animate is false */
-        return gridboard->animate;
+	if (flipped_tiles) 
+	  gtk_gridboard_paint (gridboard);
+
+        return TRUE;
 }
 
-static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard) {
-        gchar *filepath=gridboard->tileset;
-        GdkPixbuf *image;
-
-        if (! g_file_test (filepath, G_FILE_TEST_EXISTS)) {
-                g_print ("Could not find \'%s\' pixmap file\n", 
-                                filepath);
-                exit (1);
-        }
-
-        image = gdk_pixbuf_new_from_file (filepath, NULL);
-        if (image==NULL) {
-                g_warning ("loading pixmaps failed\n");
-                exit(1);
-        }
-
-        /* what happens here ? */
-        gdk_pixbuf_render_pixmap_and_mask_for_colormap (image, 
-							gdk_colormap_get_system (),
-							&(gridboard->tiles_pixmap), 
-							NULL, 127);
-  
-        gdk_pixbuf_unref (image);
-}
-
-
 /* helper functions */
 
-/* allocates memory for the array */
-static gint ** make_array(int width, int height) {
-        gint ** result;
-        int valarraysiz, ptrarraysiz;
-        int i;
-        
-        valarraysiz=width*height;       /* size of actual array */
-        ptrarraysiz=width;              /* size of pointer array */
-        result=(gint **)g_malloc(ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
-        memset(result, 0, ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
-        for (i=0; i<width; i++) {
-                result[i]=(gint *)(result+ptrarraysiz)+i*height;
-        }
-        return result;
+/* Allocates memory for an array and fills it with the byte fill. */
+static gint ** make_array(int width, int height, int fill) 
+{
+  gint ** result;
+  int valarraysiz, ptrarraysiz;
+  int i;
+  
+  valarraysiz = width*height;       /* size of actual array */
+  ptrarraysiz = width;              /* size of pointer array */
+  result = (gint **) g_malloc (ptrarraysiz*sizeof(gint *)+valarraysiz*sizeof(gint));
+  for (i=0; i<width; i++) {
+    result[i] = (gint *)(result+ptrarraysiz)+i*height;
+    memset (result[i], fill, sizeof (gint)*height);
+  }
+  return result;
 }
 
 int gtk_gridboard_count_pieces(GtkGridBoard * gridboard, int piece) {
@@ -484,24 +479,15 @@ void g_cclosure_user_marshal_VOID__INT_INT (GClosure     *closure,
 
 /* option setting functions */
 
-void gtk_gridboard_set_show_grid(GtkGridBoard *gridboard, gboolean showgrid) {
-        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
-
-        gridboard->showgrid=showgrid;
-        gtk_gridboard_paint(gridboard);
-}
-
 void gtk_gridboard_set_animate(GtkGridBoard *gridboard, gboolean animate) {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
         gridboard->animate=animate;
-        if (animate) {
-	  if (timeoutid)
-	    return;
-	  timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
-				    gtk_gridboard_flip_pixmaps,
-				    (gpointer) gridboard);
-        } else timeoutid = 0;
+	if (gridboard->timeoutid)
+	  return;
+	gridboard->timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
+					     gtk_gridboard_flip_pixmaps,
+					     (gpointer) gridboard);
 }
 
 void gtk_gridboard_set_visibility(GtkGridBoard *gridboard, gboolean visibility) {
@@ -510,12 +496,27 @@ void gtk_gridboard_set_visibility(GtkGridBoard *gridboard, gboolean visibility)
 	gridboard->visibility=visibility;
 }
 
-void gtk_gridboard_set_tileset(GtkGridBoard *gridboard, gchar * tileset) {
-        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+void gtk_gridboard_set_theme (GtkGridBoard *gridboard, gchar * name)
+{
+        GtkGridBoardTheme *t;
+	gint x, y;
 
-        gridboard->tileset=tileset;
-        gtk_gridboard_load_pixmaps(gridboard);
-        gtk_gridboard_paint(gridboard);
+        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (name[0] != '\0');
+
+	t = gtk_gridboard_themes;
+	while (t->name != NULL) {
+	  if (g_utf8_collate (t->name, name) == 0) {
+	    gridboard->theme = t;
+	    break;
+	  }
+	  t++;
+	}
+
+	for (x=0; x<gridboard->width; x++)
+	  for (y=0; y<gridboard->height; y++)
+	    gridboard->changed[x][y] = TRUE;
 }
         
 /* board altering functions */
@@ -524,12 +525,11 @@ void gtk_gridboard_set_piece(GtkGridBoard *gridboard, int x, int y,
 {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
+	if (gridboard->board[x][y] == piece)
+	  return;
+
         gridboard->board[x][y]=piece;
-        /* draw updated piece */
-        if (!gridboard->animate) {
-                gtk_gridboard_draw_pixmap(gridboard, get_pixmap_num(piece), x, y
-);
-        }
+	
 }
 
 int gtk_gridboard_get_piece(GtkGridBoard *gridboard, int x, int y) 
@@ -547,10 +547,12 @@ void gtk_gridboard_clear_selections(GtkGridBoard *gridboard)
 
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
-			gridboard->selected[x][y]=SELECTED_NONE;
+		  if (gridboard->selected[x][y] != SELECTED_NONE) {
+		    gridboard->selected[x][y] = SELECTED_NONE;
+		    gridboard->changed[x][y] = TRUE;
+		  }
 		}
 	}
-	gtk_gridboard_paint(gridboard);
 }
 
 void gtk_gridboard_clear_pieces(GtkGridBoard *gridboard) 
@@ -561,24 +563,10 @@ void gtk_gridboard_clear_pieces(GtkGridBoard *gridboard)
 
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
-			gridboard->board[x][y]=EMPTY;
+			gridboard->board[x][y] = EMPTY;
+			gridboard->changed[x][y] = TRUE;
 		}
 	}
-	gtk_gridboard_paint(gridboard);
-}
-
-void gtk_gridboard_clear_pixmaps(GtkGridBoard *gridboard) 
-{
-	int x, y;
-
-        g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
-
-	for (x=0; x<gridboard->width; x++) {
-		for (y=0; y<gridboard->height; y++) {
-			gridboard->pixmaps[x][y]=get_pixmap_num(EMPTY);
-		}
-	}
-	gtk_gridboard_paint(gridboard);
 }
 
 void gtk_gridboard_clear(GtkGridBoard *gridboard) 
@@ -587,7 +575,6 @@ void gtk_gridboard_clear(GtkGridBoard *gridboard)
 
 	gtk_gridboard_clear_selections(gridboard);
 	gtk_gridboard_clear_pieces(gridboard);
-	gtk_gridboard_clear_pixmaps(gridboard);
 	gtk_gridboard_clear_states(gridboard);
 }
 
@@ -608,7 +595,7 @@ void gtk_gridboard_save_state(GtkGridBoard *gridboard, gpointer data)
 
 	newstate = g_malloc(sizeof(StateList));
 
-	newstate->board=make_array(gridboard->width, gridboard->height);
+	newstate->board = make_array (gridboard->width, gridboard->height, 0);
 	for (x=0; x<gridboard->width; x++) {
 		for (y=0; y<gridboard->height; y++) {
 			newstate->board[x][y]=gridboard->board[x][y];
@@ -634,15 +621,16 @@ gpointer gtk_gridboard_revert_state(GtkGridBoard *gridboard)
 	
 	prevstate=gridboard->statelist->prev;
 
-	for (x=0; x<gridboard->width; x++) {
-		for (y=0; y<gridboard->width; y++) {
-			gridboard->board[x][y]=thisstate->board[x][y];
-		}
-	}
-	if (!gridboard->animate) gtk_gridboard_paint(gridboard);
-
 	gtk_gridboard_clear_selections (gridboard);
-
+	for (x=0; x<gridboard->width; x++) {
+	  for (y=0; y<gridboard->width; y++) {
+	    if (gridboard->board[x][y] != thisstate->board[x][y]) {
+	      gridboard->board[x][y] = thisstate->board[x][y];
+	      gridboard->changed[x][y] = TRUE;
+	    }
+	  }
+	}
+	
 	data=thisstate->data;
 	gridboard->statelist=prevstate;
 	g_free(thisstate->board);
