@@ -138,7 +138,7 @@ network_game_dialog_show (GtkWidget *parent_window)
   /*  Create Network Status Frame. */
   frame = games_frame_new (_("Status"));
   gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
-  status_view = gtk_label_new (_("Select a nickname and connection method\nthen click Connect."));
+  status_view = gtk_label_new (_("Select a nickname and connection method then click Connect."));
   gtk_container_add (GTK_CONTAINER (frame), status_view);
 
   /* Now for the network connection options. */
@@ -172,12 +172,12 @@ network_game_dialog_show (GtkWidget *parent_window)
   rbutton_b = gtk_radio_button_new_with_label_from_widget (
 			GTK_RADIO_BUTTON (rbutton_a),
 			_("Host local game"));
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (rbutton_b, FALSE);
 #endif
   gtk_box_pack_start (GTK_BOX(vbox2), rbutton_b, FALSE, FALSE, 0);
   game_entry = gtk_entry_new ();
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (game_entry, FALSE);
 #endif
   gtk_entry_set_max_length(GTK_ENTRY(game_entry), 12);
@@ -194,7 +194,7 @@ network_game_dialog_show (GtkWidget *parent_window)
   label = gtk_label_new_with_mnemonic (_("_Game name:"));
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), game_entry);
   gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (label, FALSE);
   gtk_widget_set_sensitive (game_entry, FALSE);
 #endif
@@ -204,7 +204,7 @@ network_game_dialog_show (GtkWidget *parent_window)
   rbutton_c = gtk_radio_button_new_with_label_from_widget (
                         GTK_RADIO_BUTTON (rbutton_b),
                         _("Join local game"));
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (rbutton_c, FALSE);
 #endif
   gtk_box_pack_start (GTK_BOX(vbox2), rbutton_c, FALSE, FALSE, 0);
@@ -223,12 +223,12 @@ network_game_dialog_show (GtkWidget *parent_window)
   gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
   
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (label, FALSE);
 #endif
   model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
   view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(model));
-#ifdef NO_HOWL
+#ifndef HAVE_ZEROCONF
   gtk_widget_set_sensitive (view, FALSE);
 #endif
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(view), FALSE);
@@ -261,7 +261,7 @@ network_game_dialog_show (GtkWidget *parent_window)
   }
 
   /* Add the players nickname entry. */
-  frame = games_frame_new ("Identification");
+  frame = games_frame_new (_("Identification"));
 
   label = gtk_label_new_with_mnemonic(_("_Nickname:"));
 
@@ -369,7 +369,7 @@ connect_cb (GtkWidget *widget)
       gtk_widget_set_sensitive(GTK_WIDGET(connect_cmd), TRUE);
     }
   } else if (network_mode == JOIN_LAN_MODE) {
-    games_network_connect (server, name);
+    games_network_connect (lan_server, name);
 
   }
 
